@@ -5,8 +5,8 @@
  *
  * Copyright (C) 2012 Mark Shelor, All Rights Reserved
  *
- * Version: 0.03
- * Mon Oct 29 04:01:06 MST 2012
+ * Version: 0.04
+ * Sun Nov 11 19:20:06 MST 2012
  *
  */
 
@@ -24,17 +24,6 @@
 #define SR64	SHA64_SHR
 #define SL64	SHA64_SHL
 
-/* mem2word: convert little-endian to 64-bit value */
-static W64 mem2word(UCHR *mem)
-{
-	UCHR *p;
-	W64 w = 0;
-
-	for (p = mem+7; p >= mem;)
-		w = (w << 8) | *p--;
-	return(w);
-}
-
 /* word2mem: write 64-bit value in little-endian order */
 static UCHR *word2mem(W64 w, UCHR *mem)
 {
@@ -46,41 +35,41 @@ static UCHR *word2mem(W64 w, UCHR *mem)
 	return(mem);
 }
 
-static W64 RC[] = {		/* Keccak round constants */
-C64(0x0000000000000001), C64(0x0000000000008082), C64(0x800000000000808a),
-C64(0x8000000080008000), C64(0x000000000000808b), C64(0x0000000080000001),
-C64(0x8000000080008081), C64(0x8000000000008009), C64(0x000000000000008a),
-C64(0x0000000000000088), C64(0x0000000080008009), C64(0x000000008000000a),
-C64(0x000000008000808b), C64(0x800000000000008b), C64(0x8000000000008089),
-C64(0x8000000000008003), C64(0x8000000000008002), C64(0x8000000000000080),
-C64(0x000000000000800a), C64(0x800000008000000a), C64(0x8000000080008081),
-C64(0x8000000000008080), C64(0x0000000080000001), C64(0x8000000080008008)
+static W64 RC[] = {	/* Keccak round constants */
+	C64(0x0000000000000001), C64(0x0000000000008082),
+	C64(0x800000000000808a), C64(0x8000000080008000),
+	C64(0x000000000000808b), C64(0x0000000080000001),
+	C64(0x8000000080008081), C64(0x8000000000008009),
+	C64(0x000000000000008a), C64(0x0000000000000088),
+	C64(0x0000000080008009), C64(0x000000008000000a),
+	C64(0x000000008000808b), C64(0x800000000000008b),
+	C64(0x8000000000008089), C64(0x8000000000008003),
+	C64(0x8000000000008002), C64(0x8000000000000080),
+	C64(0x000000000000800a), C64(0x800000008000000a),
+	C64(0x8000000080008081), C64(0x8000000000008080),
+	C64(0x0000000080000001), C64(0x8000000080008008)
 };
 
-static int R[][5] = {		/* Keccak rotation offsets */
-	{0,  36,  3, 41, 18}, {1,  44, 10, 45,  2}, {62,  6, 43, 15, 61},
-	{28, 55, 25, 21, 56}, {27, 20, 39,  8, 14}
-};
-
-/* rot: rotate 64-bit word left by n bit positions */
-#define rot(w, n) (SR64((w), (64 - (n))) | SL64((w), (n)))
+/* ROTL: rotate 64-bit word left by n bit positions */
+#define ROTL(w, n) (SR64((w), (64 - (n))) | SL64((w), (n)))
 
 /* keccak_f: apply KECCAK-f[1600] permutation for 24 rounds */
 static void keccak_f(W64 A[][5])
 {
 	int i;
-	for (i = 0; i < 24; i++) {
+	W64 *rc = RC;
+	for (i = 0; i < 24; i++, rc++) {
 		W64 B[5][5], C[5], D[5];
 		C[0] = A[0][0]^A[0][1]^A[0][2]^A[0][3]^A[0][4];
 		C[1] = A[1][0]^A[1][1]^A[1][2]^A[1][3]^A[1][4];
 		C[2] = A[2][0]^A[2][1]^A[2][2]^A[2][3]^A[2][4];
 		C[3] = A[3][0]^A[3][1]^A[3][2]^A[3][3]^A[3][4];
 		C[4] = A[4][0]^A[4][1]^A[4][2]^A[4][3]^A[4][4];
-		D[0] = C[4] ^ rot(C[1], 1);
-		D[1] = C[0] ^ rot(C[2], 1);
-		D[2] = C[1] ^ rot(C[3], 1);
-		D[3] = C[2] ^ rot(C[4], 1);
-		D[4] = C[3] ^ rot(C[0], 1);
+		D[0] = C[4] ^ ROTL(C[1], 1);
+		D[1] = C[0] ^ ROTL(C[2], 1);
+		D[2] = C[1] ^ ROTL(C[3], 1);
+		D[3] = C[2] ^ ROTL(C[4], 1);
+		D[4] = C[3] ^ ROTL(C[0], 1);
 		A[0][0] ^= D[0];
 		A[0][1] ^= D[0];
 		A[0][2] ^= D[0];
@@ -106,31 +95,31 @@ static void keccak_f(W64 A[][5])
 		A[4][2] ^= D[4];
 		A[4][3] ^= D[4];
 		A[4][4] ^= D[4];
-		B[0][0] = rot(A[0][0], R[0][0]);
-		B[1][3] = rot(A[0][1], R[0][1]);
-		B[2][1] = rot(A[0][2], R[0][2]);
-		B[3][4] = rot(A[0][3], R[0][3]);
-		B[4][2] = rot(A[0][4], R[0][4]);
-		B[0][2] = rot(A[1][0], R[1][0]);
-		B[1][0] = rot(A[1][1], R[1][1]);
-		B[2][3] = rot(A[1][2], R[1][2]);
-		B[3][1] = rot(A[1][3], R[1][3]);
-		B[4][4] = rot(A[1][4], R[1][4]);
-		B[0][4] = rot(A[2][0], R[2][0]);
-		B[1][2] = rot(A[2][1], R[2][1]);
-		B[2][0] = rot(A[2][2], R[2][2]);
-		B[3][3] = rot(A[2][3], R[2][3]);
-		B[4][1] = rot(A[2][4], R[2][4]);
-		B[0][1] = rot(A[3][0], R[3][0]);
-		B[1][4] = rot(A[3][1], R[3][1]);
-		B[2][2] = rot(A[3][2], R[3][2]);
-		B[3][0] = rot(A[3][3], R[3][3]);
-		B[4][3] = rot(A[3][4], R[3][4]);
-		B[0][3] = rot(A[4][0], R[4][0]);
-		B[1][1] = rot(A[4][1], R[4][1]);
-		B[2][4] = rot(A[4][2], R[4][2]);
-		B[3][2] = rot(A[4][3], R[4][3]);
-		B[4][0] = rot(A[4][4], R[4][4]);
+		B[0][0] = A[0][0];
+		B[1][3] = ROTL(A[0][1], 36);
+		B[2][1] = ROTL(A[0][2], 3);
+		B[3][4] = ROTL(A[0][3], 41);
+		B[4][2] = ROTL(A[0][4], 18);
+		B[0][2] = ROTL(A[1][0], 1);
+		B[1][0] = ROTL(A[1][1], 44);
+		B[2][3] = ROTL(A[1][2], 10);
+		B[3][1] = ROTL(A[1][3], 45);
+		B[4][4] = ROTL(A[1][4], 2);
+		B[0][4] = ROTL(A[2][0], 62);
+		B[1][2] = ROTL(A[2][1], 6);
+		B[2][0] = ROTL(A[2][2], 43);
+		B[3][3] = ROTL(A[2][3], 15);
+		B[4][1] = ROTL(A[2][4], 61);
+		B[0][1] = ROTL(A[3][0], 28);
+		B[1][4] = ROTL(A[3][1], 55);
+		B[2][2] = ROTL(A[3][2], 25);
+		B[3][0] = ROTL(A[3][3], 21);
+		B[4][3] = ROTL(A[3][4], 56);
+		B[0][3] = ROTL(A[4][0], 27);
+		B[1][1] = ROTL(A[4][1], 20);
+		B[2][4] = ROTL(A[4][2], 39);
+		B[3][2] = ROTL(A[4][3], 8);
+		B[4][0] = ROTL(A[4][4], 14);
 		A[0][0] = B[0][0] ^ (~B[1][0] & B[2][0]);
 		A[0][1] = B[0][1] ^ (~B[1][1] & B[2][1]);
 		A[0][2] = B[0][2] ^ (~B[1][2] & B[2][2]);
@@ -156,7 +145,7 @@ static void keccak_f(W64 A[][5])
 		A[4][2] = B[4][2] ^ (~B[0][2] & B[1][2]);
 		A[4][3] = B[4][3] ^ (~B[0][3] & B[1][3]);
 		A[4][4] = B[4][4] ^ (~B[0][4] & B[1][4]);
-		A[0][0] ^= RC[i];
+		A[0][0] ^= *rc;
 	}
 }
 
@@ -166,13 +155,13 @@ static void sha3(SHA3 *s, UCHR *block)
 	int i, x, y;
 	W64 P0[5][5];
 
-	for (i = 0; i < s->blocksize/64; i++)
-		P0[i%5][i/5] = mem2word(block+i*8);
+	for (i = 0; i < s->blocksize/64; i++, block += 8)
+		MEM2WORD(&P0[i%5][i/5], block);
 	for (x = 0; x < 5; x++)
 		for (y = 0; y < 5; y++) {
 			if (x + y*5 >= s->blocksize/64)
 				break;
-			s->S[x][y] = s->S[x][y] ^ P0[x][y];
+			s->S[x][y] ^= P0[x][y];
 		}
 	keccak_f(s->S);
 }
@@ -225,7 +214,7 @@ SHA3 *shaopen(int alg)
 	SHA3 *s;
 
 	if (alg != SHA3_0 && alg != SHA3_224 && alg != SHA3_256 &&
-		alg != SHA3_384    && alg != SHA3_512)
+		alg != SHA3_384 && alg != SHA3_512)
 		return(NULL);
 	SHA3_newz(0, s, 1, SHA3);
 	if (s == NULL)
